/*
 * STM32F3-Discovery demo
 * 2013, Kai O.
 */

#include <stm32f30x_usart.h>
#include <stm32f30x_gpio.h>
#include <stm32f30x_rcc.h>
#include <stm32f30x.h>

#include "serial.h"
#include "printf.h"

static void sendchar(unsigned char c)  __attribute__((always_inline));
static void UARTwrite(void *usrarg, char c);

void uartInit()
{
   RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
   GPIO_PinAFConfig(GPIOA, GPIO_PinSource9, GPIO_AF_7);
   GPIO_PinAFConfig(GPIOA, GPIO_PinSource10, GPIO_AF_7);

   GPIO_InitTypeDef uartGpio;
   uartGpio.GPIO_Mode = GPIO_Mode_AF;
   uartGpio.GPIO_OType = GPIO_OType_PP;
   uartGpio.GPIO_Pin = GPIO_Pin_9 | GPIO_Pin_10;
   uartGpio.GPIO_PuPd = GPIO_PuPd_UP;
   uartGpio.GPIO_Speed = GPIO_Speed_50MHz;

   GPIO_Init(GPIOA, &uartGpio);
   RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);

   USART_InitTypeDef USART_InitStruct;

   USART_InitStruct.USART_BaudRate = 115200;
   USART_InitStruct.USART_WordLength = USART_WordLength_8b;
   USART_InitStruct.USART_StopBits = USART_StopBits_1;
   USART_InitStruct.USART_Parity = USART_Parity_No ;
   USART_InitStruct.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
   USART_InitStruct.USART_HardwareFlowControl = USART_HardwareFlowControl_None;

   USART_Init(USART1, &USART_InitStruct);

   USART_Cmd(USART1, ENABLE);

   init_printf(0, UARTwrite);

   printf("USART ready\r\n");
}

static void sendchar(unsigned char c)
{
   /* Wait until the transmitter is ready */
   while (! (USART1->ISR & USART_ISR_TXE));

   /* Transmit the character using USART1 */
   USART1->TDR = c;
}

void sout(char *s)
{
   while (*s)
   {
      sendchar(*s);
      s++;
   }
}

static void UARTwrite(void *usrarg, char c)
{
   usrarg = usrarg;
   sendchar(c);
}
